# This is a practice program for people learning to use Numpy and Scipy.
# Copyright Sam Kennerly 2010 (Creative Commons Attribution/Noncommercial/Share-Alike 3.0)
# http://creativecommons.org/licenses/by-nc-sa/3.0/

# TO RUN THIS PROGRAM:
# Open a terminal and change to the directory containing this file, then type "python Demo.py" .
# The file oscillators.dat must be in the same directory.  (It contains all the data!)


from numpy import *		# general NumPy stuff
from scipy import *		# general SciPy stuff
from pylab import *		# general PyLab stuff
from numpy.fft import *		# stuff for fast Fourier transforms
import math				# general math stuff
	
# First, look for the file oscillators.dat.
print "Opening file oscillators.dat ..."

# Open the file in "read-only" mode.
inputfile = open("oscillators.dat","r")

# Now we'll make a big list of all the numbers in the file.
inputlist = inputfile.read().split()

# Technically, they aren't numbers yet - they're character strings.
# Let's make them into a big array of high-precision 64-bit floating point numbers.
inputarray = array(float64(inputlist))

# Now just to be safe and save memory, we'll close the file and delete inputlist.
del inputlist
del inputfile

# This file was (presumably) made by the program oscillators.py .
# Based on how we stored the data then, we know that inputarray looks like this:
# [t0, x0, y0, z0, vx0, vy0, vz0, t1, x1, y1, z1, vx1, vy1, vz1, t2, ...]
# Let's break it into its 7 columns using a "slice by 7" trick:
timesteps = inputarray[0::7]
harmonic_x = inputarray[1::7]
vanderpol_x = inputarray[2::7]
duffing_x = inputarray[3::7]
harmonic_v = inputarray[4::7]
vanderpol_v = inputarray[5::7]
duffing_v = inputarray[6::7]

# Each of those arrays is size 1 by (a lot).  Let's figure out how much (a lot) is:
num_points = len(timesteps)

# Now we use the fft function from NumPy to create new arrays:
HARMONIC_X = fft(harmonic_x)
VANDERPOL_X = fft(vanderpol_x)
DUFFING_X = fft(duffing_x)
HARMONIC_V = fft(harmonic_v)
VANDERPOL_V = fft(vanderpol_v)
DUFFING_V = fft(duffing_v)
# These are arrays of complex numbers.  Also, they're sorted in a weird order.
# It's called "FFT standard order" if you want to look it up.

# Use fftfreq(num_points) to make an array of all the frequencies in order.
# This will be handy when we want to plot power spectra versus frequency.
freq_array = fftfreq(num_points)

# By the way, the "inverse FFT" function is called ifft and it works just like fft.
# Watch out - coefficients must still be in "standard order."


# Turn on Pylab interactive mode so the plot window will be visible while the program runs.
ion()


# We'll plot the power spectrum of x and v for all the oscillators.
# We'll put 6 subplots on one page just to see how it's done.  For each one,
# y-axis is [abs()]^2 of FFT, x-axis is frequency, and both axes are log scale.

figure(1)
subplots_adjust(wspace=0.2, hspace=0.5)		# Make some extra space between plots.

subplot(3,2,1)	# We'll show 3 rows, 2 columns of plots.  This is plot #1.
loglog(freq_array, abs(HARMONIC_X)**2)
title("Harmonic x")
ylabel("Power")
grid(True)

subplot(3,2,2)	# This is plot #2.
loglog(freq_array, abs(HARMONIC_V)**2)
title("Harmonic v")
grid(True)

subplot(3,2,3)	# This is plot #3.
loglog(freq_array, abs(VANDERPOL_X)**2)
title("van der Pol x")
ylabel("Power")
grid(True)

subplot(3,2,4)
loglog(freq_array, abs(VANDERPOL_V)**2)
title("van der Pol v")		
grid(True)

subplot(3,2,5)
loglog(freq_array, abs(DUFFING_X)**2)
title("Duffing x")
grid(True)
ylabel("Power")
xlabel("Frequency (cycles/sample)")

subplot(3,2,6)
loglog(freq_array, abs(DUFFING_V)**2)
title("Duffing v")
grid(True)
xlabel("Frequency (cycles/sample)")

# Let's save all these plots as one picture.
savefig('Spectra.png')

# These plots disappear as soon as the program closes.  Here's a trick to keep them open:
press_any_key = raw_input("\nPress Enter to quit.")
ioff()		# Turn off interactive mode.